'use strict';

const
	path = require('path'),
	phantomjs = require('phantomjs'),
	cp = require('child_process'),
	base64stream = require('base64-stream'),
	highland = require('highland'),
	shared = require('./shared'),
	config = require('./config');

module.exports = function(testURL){
	return highland([testURL])
		.map(serializeSVG)
		.flatten()
		.through((stream) => {
			let promise = shared
				.streamToPromise(stream)
				.then(shared.joinBuffer)
				.then(xmlToBase64);
			return highland(promise);
		})
		.flatten()
		.pipe(base64stream.decode());
};

function serializeSVG(testURL){
	let
		args = [
			path.join(__dirname, 'svg-to-xml.js'),
			JSON.stringify(config),
			testURL
		],
		child = cp.spawn(phantomjs.path, args);
	return highland(child.stdout);
}

function xmlToBase64(buffer){
	let
		args = [
			path.join(__dirname, 'xml-to-base64.js'),
			JSON.stringify(config),
			buffer.toString()
		],
		child = cp.spawn(phantomjs.path, args);
	return highland(child.stdout);
}
